package fumdb.servlet;

import java.text.DateFormat;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import fumdb.domain.Musician;
import fumdb.domain.Record;
import fumdb.domain.RecordCompany;
import fumdb.domain.Song;
import fumdb.domain.Tracklist;

/**
 * @author Richard Cyganiak
 */
public class RecordPage extends DatabasePage {

	private String requestedRecord;
	private Record record;
	private Tracklist tracklist;
	private RecordCompany recordCompany;
	private Vector functions;
	private Map musicians;

	public RecordPage(String requestedRecord) {
		this.requestedRecord = requestedRecord;
	}

	protected void init() throws RequestException {
		try {
			int id = Integer.parseInt(requestedRecord);
			record = getDb().getRecord(id);
		} catch (NumberFormatException e) {
			throw new RequestException("Unbekannte Tontrger-ID: " + requestedRecord);
		}
		if (record == null) {
			throw new RequestException("Unbekannte Tontrger-ID: '" + requestedRecord);
		}
		tracklist = record.getTracklist();
		recordCompany = record.getRecordCompany();
		functions = record.getAllFunctions();
		musicians = new HashMap();
		Iterator it = tracklist.iterator();
		while (it.hasNext()) {
			Song each = (Song) it.next();
			musicians.put(each, each.getMusician());
		}
	}

	protected String getTitle() {
		return "Tontrger: " + record.getFullTitle();
	}

	protected String getHeadline() {
		return record.getFullTitle();
	}

	protected void printBody() {
		boolean anythingPrinted = false;
		if (record.hasMainArtist()) {
			printMusicianInfo();
			anythingPrinted = true;
		}
		if (record.getPublishedDate() != null || recordCompany != null) {
			printPublishedInfo();
			anythingPrinted = true;
		}
		if (tracklist.getGenres().size() > 0) {
			printGenres();
			anythingPrinted = true;
		}
		if (tracklist.size() > 0) {
			printTracksOverview();
			printTrackList();
			anythingPrinted = true;
		}
		if (functions.size() > 0) {
//			printFunctions();
//			anythingPrinted = true;
		}
		if (!anythingPrinted) {
			p("Keine weiteren Informationen vorhanden.");
		}
	}

	private void printMusicianInfo() {
		p("Knstler: <strong>" + link("mid=" + record.getMusicianID(), record.getMusicianName()) + "</strong>");
	}
	
	private void printPublishedInfo() {
		String html = "Verffentlicht:";
		if (record.getPublishedDate() != null) {
			DateFormat format = DateFormat.getDateInstance(DateFormat.LONG);
			html += " " + format.format(record.getPublishedDate());
		}
		if (recordCompany != null) {
			html += " von " + recordCompany.getName();
			if (recordCompany.getURL() != null) {
				html += " [" + xlink(recordCompany.getURL()) + "]";
			}
		}
		p(html);
	}

	private void printGenres() {
		String html = null;
		Iterator it = tracklist.getGenres().iterator();
		while (it.hasNext()) {
			if (html != null) {
				html += ", ";
			} else {
				html = "";
			}
			html += getGenreLink(it.next().toString());
		}
		p("Genre: " + html);
	}

	private void printTracksOverview() {
		int length = tracklist.getLength();
		int count = tracklist.size();
		if (length > 0) {
			p(count + " Tracks, " + getFormattedLength(length) + " min");
		} else {
			p(count + " Tracks");
		}
	}
	
	private String getFormattedLength(int length) {
		String hours = Integer.toString(length / 3600);
		String minutes = Integer.toString((length / 60) % 60);
		String seconds = Integer.toString(length % 60);
		if (seconds.length() == 1) {
			seconds = "0" + seconds;
		}
		if (!hours.equals("0") && minutes.length() == 1) {
			minutes = "0" + minutes;
		}
		if (hours.equals("0")) {
			return minutes + ":" + seconds;
		} else {
			return hours + ":" + minutes + ":" + seconds;
		}
	}

	private void printTrackList() {
		out("<table>");
		boolean isCompilation = tracklist.isCompilation();
		for (int i = 1; i < tracklist.size(); i++) {
			Song song = tracklist.getSong(i);
			int length = tracklist.getLength(i);
			String lengthString = (length == 0) ? "" : getFormattedLength(length);
			out("<tr><td align=\"right\">" + i + ".</td>");
			if (isCompilation) {
				Musician musician = (Musician) musicians.get(song);
				out("<td>" + getMusicianLink(musician.getName(), musician.getId()) + " - " + song.getName() + "</td>");
			} else {
				out("<td>" + song.getName() + "</td>");
			}
			out("<td align=\"right\">" + lengthString + "</td></tr>");
		}
		out("</table>");
	}

/*
	private void printMusicians() {
		h2("Knstler und Bands");
		Iterator it = musicians.iterator();
		out("<p>");
		while (it.hasNext()) {
			out(getMusicianHtml((Musician) it.next()));
		}
		out("</p>");
	}
	
	private String getMusicianHtml(Musician musician) {
		return link("mid=" + musician.getId(), musician.getName()) + "<br>";
	}
	
	private void printRecords() {
		h2("Tontrger");
		Iterator it = records.keySet().iterator();
		out("<p>");
		while (it.hasNext()) {
			Record each = (Record) it.next();
			Integer genreQuota = (Integer) records.get(each);
			out(getRecordHtml(each, genreQuota.intValue()));
		}
		out("</p>");
	}

	private String getRecordHtml(Record record, int genreQuota) {
		return link("tid=" + record.getId(), record.getFullTitle()) + " (" + genreQuota + "%)<br>";
	}

	private void printEmpty() {
		out("In diesem Genre haben wir keine Eintrge.");
	}
*/
}